#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	/// <summary>
    /// Calculates Exponential Volume Weighted Average Price
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
	public class mtExpVWAP : Indicator
	{
		#region Variables
		
		private Series<double> num;
		private Series<double> den;
		private double alpha = 0;
		
		#endregion

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Exponential Volume Weighted Average Price";
				Name										= "mtExpVWAP";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				Period					= 14;
				AddPlot(Brushes.Blue, "VWEMA");
			}
			else if (State == State.Configure)
			{
			}
			else if (State == State.DataLoaded)
			{				
				alpha = 1-2/((double)Period+1);
				
				num = new Series<double>(this);
				den = new Series<double>(this);
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar==0)
			{
				num[0] = (1-alpha)*Volume[0]*Input[0];
				den[0] = (1-alpha)*Volume[0];
			}
			else
			{
				num[0] = alpha*num[1]+(1-alpha)*Volume[0]*Input[0];
				den[0] = alpha*den[1]+(1-alpha)*Volume[0];
			}
			
			if (den[0]!=0)
				VEMA[0] = num[0]/den[0];
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Period", Order=1, GroupName="Parameters")]
		public int Period
		{ get; set; }

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> VEMA
		{
			get { return Values[0]; }
		}
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private mtExpVWAP[] cachemtExpVWAP;
		public mtExpVWAP mtExpVWAP(int period)
		{
			return mtExpVWAP(Input, period);
		}

		public mtExpVWAP mtExpVWAP(ISeries<double> input, int period)
		{
			if (cachemtExpVWAP != null)
				for (int idx = 0; idx < cachemtExpVWAP.Length; idx++)
					if (cachemtExpVWAP[idx] != null && cachemtExpVWAP[idx].Period == period && cachemtExpVWAP[idx].EqualsInput(input))
						return cachemtExpVWAP[idx];
			return CacheIndicator<mtExpVWAP>(new mtExpVWAP(){ Period = period }, input, ref cachemtExpVWAP);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.mtExpVWAP mtExpVWAP(int period)
		{
			return indicator.mtExpVWAP(Input, period);
		}

		public Indicators.mtExpVWAP mtExpVWAP(ISeries<double> input , int period)
		{
			return indicator.mtExpVWAP(input, period);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.mtExpVWAP mtExpVWAP(int period)
		{
			return indicator.mtExpVWAP(Input, period);
		}

		public Indicators.mtExpVWAP mtExpVWAP(ISeries<double> input , int period)
		{
			return indicator.mtExpVWAP(input, period);
		}
	}
}

#endregion
